@@ -1,11 +1,11 @@
 package App::jl;
 use strict;
 use warnings;
-use JSON qw/decode_json to_json/;
+use JSON qw//;
 use Sub::Data::Recursive;
 use Getopt::Long qw/GetOptionsFromArray/;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 my $MAX_DEPTH = 10;
 
@@ -13,8 +13,11 @@ sub new {
     my $class = shift;
     my @argv  = @_;
 
+    my $opt = $class->_parse_opt(@argv);
+
     bless {
-        _opt => $class->_parse_opt(@argv),
+        _opt  => $opt,
+        _json => JSON->new->utf8->pretty(!$opt->{no_pretty})->canonical(1),
     }, $class;
 }
 
@@ -28,6 +31,10 @@ sub run {
     my ($self) = @_;
 
     while (my $orig_line = <STDIN>) {
+        if ($orig_line !~ m!^[\[\{]!) {
+            print $orig_line;
+            next;
+        }
         print $self->process($orig_line);
     }
 }
@@ -35,60 +42,46 @@ sub run {
 sub process {
     my ($self, $line) = @_;
 
-    my $decoded;
-    eval {
-        $decoded = decode_json($line);
+    my $decoded = eval {
+        $self->{_json}->decode($line);
     };
     if ($@) {
         return $line;
     }
     else {
-        $self->_recursive_split($decoded) if $self->opt('x');
+        Sub::Data::Recursive->invoke(\&_split => $decoded) if $self->opt('x');
         $self->{_depth} = $self->opt('depth');
         $self->_recursive_decode_json($decoded);
-        return to_json($decoded, {pretty => !$self->opt('no_pretty')});
+        return $self->{_json}->encode($decoded);
     }
 }
 
-sub _recursive_split {
-    my ($self, $hash) = @_;
+sub _split {
+    my $line = $_[0];
 
-    Sub::Data::Recursive->invoke(
-        sub {
-            my $line = $_[0];
-
-            chomp $line;
-
-            if ($line =~ m![\t\r\n]!) {
-                my @elements = split /[\t\r\n]/, $line;
-                $_[0] = \@elements;
-            }
-        },
-        $hash
-    );
+    if ($line =~ m![\t\r\n]!) {
+        chomp $line;
+        my @elements = split /[\t\r\n]/, $line;
+        $_[0] = \@elements;
+    }
 }
 
 sub _recursive_decode_json {
     my ($self, $hash) = @_;
 
-    Sub::Data::Recursive->invoke(
-        sub {
-            my $line = $_[0];
-
-            if ($self->{_depth} > 0) {
-                my $h;
-                eval {
-                    $h = decode_json($line);
-                };
-                if (!$@) {
-                    $self->{_depth}--;
-                    $_[0] = $h;
-                    $self->_recursive_decode_json($_[0]);
-                }
+    Sub::Data::Recursive->invoke(sub {
+        if ($self->{_depth} > 0) {
+            my $orig = $_[0];
+            my $decoded = eval {
+                $self->{_json}->decode($orig);
+            };
+            if (!$@) {
+                $self->{_depth}--;
+                $_[0] = $decoded;
+                $self->_recursive_decode_json($_[0]); # recursive calling
             }
-        },
-        $hash
-    );
+        }
+    } => $hash);
 }
 
 sub _parse_opt {
